#if UNITY_IOS

using UnityEditor;
using UnityEditor.Callbacks;
using UnityEditor.iOS.Xcode;
using System.IO;

public class BuildPostProcess {

	//このスクリプトを使うか
	private static bool useThis = true;
	//iOS9 SDK以上か?
	private static bool is_iOS9orOver = true;

  // cocoapodsに頼らないFramework追加
	private static bool use_Nend = false; //nendを有効にするか
  private static bool useAdmob = true;  //admobを有効にするか
  private static bool useSocial = true; //Socialconnectorなど

	// ビルド時に実行される
	[PostProcessBuild]
	public static void OnPostprocessBuild(BuildTarget buildTarget, string path) {

		if (buildTarget == BuildTarget.iOS) {
			if(!useThis){return;}

			string projPath = path + "/Unity-iPhone.xcodeproj/project.pbxproj";
			PBXProject proj = new PBXProject();
			proj.ReadFromString(File.ReadAllText(projPath));
      string target = proj.TargetGuidByName("Unity-iPhone");

			string plistPath = Path.Combine (path, "Info.plist");
			var plist = new PlistDocument ();
			plist.ReadFromFile (plistPath);

			string frameworkSuffix = (is_iOS9orOver) ? "tbd" : "dylib";


			proj.AddFrameworkToProject(target, "iAd.framework", false);
      proj.AddFrameworkToProject(target, "UserNotifications.framework", false);
      if (useAdmob){
          proj.AddFrameworkToProject(target, "CoreGraphics.framework", false);
          proj.AddFrameworkToProject(target, "EventKit.framework", false);
          proj.AddFrameworkToProject(target, "EventKitUI.framework", false);
          proj.AddFrameworkToProject(target, "MessageUI.framework", false);
          proj.AddFrameworkToProject(target, "StoreKit.framework", false);
          proj.AddFrameworkToProject(target, "Social.framework", false);
      }
      if(useSocial){
          proj.AddFrameworkToProject(target, "MessageUI.framework", false);
          proj.AddFrameworkToProject(target, "Security.framework", false);
          proj.AddFrameworkToProject(target, "Social.framework", false);
      }
			//nend
			if (use_Nend) {
				proj.AddFrameworkToProject(target, "AdSupport.framework", false);
				proj.AddFrameworkToProject(target, "Security.framework", false);
				proj.AddFrameworkToProject(target, "ImageIO.framework", false);
				proj.AddFrameworkToProject(target, "AVFoundation.framework", false);
				proj.AddFrameworkToProject(target, "CoreMedia.framework", false);
				proj.AddFrameworkToProject(target, "SystemConfiguration.framework", false);
				proj.AddFrameworkToProject(target, "WebKit.framework", true);  //Optional
				proj.AddFrameworkToProject(target, "CoreLocation.framework", false);
				proj.AddFrameworkToProject(target, "CoreMotion.framework", false);
				proj.AddFrameworkToProject(target, "CoreTelephony.framework", false);
        proj.AddFrameworkToProject(target, "AudioToolbox.framework", false);
        proj.AddBuildProperty(target, "HEADER_SEARCH_PATHS", "$(PROJECT_DIR)/Frameworks/NendAd/iOS/**");
      }

      proj.SetBuildProperty(target, "ENABLE_BITCODE", "NO");

      // フレームワーク検索パスの設定
      proj.AddBuildProperty(target, "FRAMEWORK_SEARCH_PATHS", "$(inherited)");
      proj.AddBuildProperty(target, "HEADER_SEARCH_PATHS", "$(inherited)");
			proj.AddBuildProperty(target, "OTHER_CFLAGS", "$(inherited)");
			proj.AddBuildProperty(target, "OTHER_LDFLAGS", "$(inherited)");
			// Frameworks/Plugins/iOS/**/*.framework
			// Use ** to make search setting in Xcode recursive
			proj.AddBuildProperty(target, "FRAMEWORK_SEARCH_PATHS", "$(PROJECT_DIR)/Frameworks/Plugins/iOS/**");

      proj.AddBuildProperty(target, "HEADER_SEARCH_PATHS", "$(PROJECT_DIR)/Frameworks");
      proj.AddBuildProperty(target, "HEADER_SEARCH_PATHS", "$(PROJECT_DIR)/Frameworks/Plugins/iOS/**");
      

			//エクスポート時に文字列のパスが加えられること(Unity由来の事象)への対応
			proj.UpdateBuildProperty (target, "HEADER_SEARCH_PATHS",
				new string[]{"$(SRCROOT)/Classes", "$(SRCROOT)"}, new string[]{"\"$(SRCROOT)/Classes\"", "\"$(SRCROOT)\""});
			proj.UpdateBuildProperty (target, "LIBRARY_SEARCH_PATHS",
				new string[]{"$(SRCROOT)/Classes", "$(SRCROOT)"}, new string[]{"\"$(SRCROOT)/Classes\"", "\"$(SRCROOT)\""});


			// Set a custom link flag
			proj.AddBuildProperty(target, "OTHER_LDFLAGS", "-ObjC");
			proj.AddBuildProperty(target, "OTHER_LDFLAGS", "-fobjc-arc");

      // 4. Include Podfile into the project root folder.
      foreach (string podFilePath in PodFilePaths)
        CopyAndReplaceFile(podFilePath, Path.Combine(path, Path.GetFileName(podFilePath)));

      File.WriteAllText(projPath, proj.WriteToString());
			plist.WriteToFile (plistPath);

      // <UNITY 4.6.6p2 - bugfix>
      // Comment the 'StartPodsProcess ()' method call and build the project again in case you receive linker errors
      // when building the Xcode project, this may happen due to a unity bug when building for ARM64 architecture.
      // Note that in this case you'll need to invoke `pods.command` manually (just double click it on Finder).
      StartPodsProcess (path);
    }
  }

#region Private methods

  internal static void CopyAndReplaceFile(string srcPath, string dstPath) {
    if (File.Exists(dstPath))
      File.Delete(dstPath);

    File.Copy(srcPath, dstPath);
  }

  internal static void CopyAndReplaceDirectory(string srcPath, string dstPath) {
    if (Directory.Exists(dstPath))
      Directory.Delete(dstPath);

    if (File.Exists(dstPath))
      File.Delete(dstPath);

    Directory.CreateDirectory(dstPath);

    foreach (var file in Directory.GetFiles(srcPath))
      File.Copy(file, Path.Combine(dstPath, Path.GetFileName(file)));

    foreach (var dir in Directory.GetDirectories(srcPath))
      CopyAndReplaceDirectory(dir, Path.Combine(dstPath, Path.GetFileName(dir)));
  }

  internal static void CopyDirectory(string srcPath, string dstPath) {
    if (!Directory.Exists(dstPath))
      Directory.CreateDirectory(dstPath);

    foreach (var file in Directory.GetFiles(srcPath))
      File.Copy(file, Path.Combine(dstPath, Path.GetFileName(file)));

    foreach (var dir in Directory.GetDirectories(srcPath))
      CopyAndReplaceDirectory(dir, Path.Combine(dstPath, Path.GetFileName(dir)));
  }

  static void StartPodsProcess(string path) {
    var proc = new System.Diagnostics.Process();
    proc.StartInfo.FileName = Path.Combine(path, OpenPodsFileName);
    proc.Start();
  }

#endregion

#region Paths

  static string[] PodFilePaths {
    get {
      return new[] {
                Path.Combine (PodFolderPath, "Podfile"),
                Path.Combine (PodFolderPath, "pods.command"),
                Path.Combine (PodFolderPath, OpenPodsFileName)
            };
    }
  }

  static string OpenPodsFileName {
    get {
      return "open_pods.command";
    }
  }



  static string PodFolderPath {
    get {
      return Path.Combine(XCodeFilesFolderPath, "Pod/");
    }
  }


  static string StringsFolderPath {
    get {
      return Path.Combine(XCodeFilesFolderPath, "Strings/");
    }
  }

  static string XCodeFilesFolderPath {
    get {
      return Path.Combine(UnityProjectRootFolder, "XcodeFiles/");
    }
  }

  static string UnityProjectRootFolder {
    get {
      return ".";
    }
  }

#endregion
}
#endif